 /***************************************************************************
 *   Copyright (C) 2006 by LZ   *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "modcond.h"
#include <qtango.h>
#include <QColor>
#include <QMessageBox>
#include <QComboBox>
#include "tlistwidget.h"

// RESIZE SUPPORT
#include <QDesktopWidget>
double resizeFactorX, resizeFactorY, resizeFactorM;

modcond::modcond(QWidget *parent) : QWidget(parent)
{
	ui.setupUi(this);

	// disable autorefresh on hide event
	foreach(QWidget *w, this->findChildren<QWidget *>()) {
	        QTangoComProxyReader * reader = dynamic_cast<QTangoComProxyReader *>(w);
	        if(reader)
#ifdef QTANGOLIB_GET_VERSION
	reader->enableHideEvents(false);
#else
	reader->setHideEventEnabled(false);
#endif
	} 
	QSettings settings("fermi", "mod");
	if (settings.contains("imgpath")) {
		imgpath = settings.value("imgpath").toString();
	}
	else {
		imgpath = "../img";
	}
	if (!imgpath.endsWith("/")) {
		imgpath = imgpath + "/";
	}
	modNum = qApp->arguments()[1];
	modNum.remove("//");
	condServer = 0;
	ui.VacuumRecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VacuumRecoveryTime");
	/*
	if (qApp->arguments().size()<=2 || !qApp->arguments()[2].contains("refl")) {
		ui.tabWidget->setTabEnabled(3,false);
		ui.VacuumRecoveryTime->hide();
		ui.VacuumRecoveryTime_label->hide();
		ui.VacuumRecoveryTime_unit->hide();
	}
	else {
		ui.VacuumRecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VacuumRecoveryTime");
		ui.VacuumRecoveryTime->show();
		ui.VacuumRecoveryTime_label->show();
		ui.VacuumRecoveryTime_unit->show();
	}
	*/
	if (!modNum.compare("05")) {
		if (connectDevice("k"+modNum+"/mod/modcond_k"+modNum+".01", &condServer) < 0) {
			return;
		}
	}
	else if (!modNum.compare("ST")) {
		if (connectDevice("kst/mod/modcond_kst.01", &condServer) < 0) {
			return;
		}
	}
	else if (!modNum.compare("XS")) {
		if (connectDevice("kxs/mod/modcond_kxs.01", &condServer) < 0) {
			return;
		}
	}
	else if (!modNum.compare("16")) {
		if (connectDevice("k16/mod/modcond_k16.01", &condServer) < 0) {
			return;
		}
		ui.VacuumRecoveryTime->setSource("k16/mod/modcond_k16.01/VacuumRecoveryTime");
	}
	else if (connectDevice("k"+modNum+"/mod/modcond_k"+modNum+".01", &condServer) < 0) {
		return;
	}
	Tango::DeviceData pumpData;
	vector<string> pump;
	try {
		pumpData = condServer->command_inout("GetVacuumGaugesNames");
		pumpData >> pump;
		ui.Pump_label_1->setText(pump.size()>0? pump[0].c_str(): "");
		ui.Pump_label_2->setText(pump.size()>1? pump[1].c_str(): "");
		ui.Pump_label_3->setText(pump.size()>2? pump[2].c_str(): "");
		ui.Pump_label_4->setText(pump.size()>3? pump[3].c_str(): "");
		ui.Pump_label_5->setText(pump.size()>4? pump[4].c_str(): "");
		ui.Pump_label_6->setText(pump.size()>5? pump[5].c_str(): "");
		ui.Pump_label_7->setText(pump.size()>6? pump[6].c_str(): "");
		ui.Pump_label_8->setText(pump.size()>7? pump[7].c_str(): "");
		ui.Pump_label_9->setText(pump.size()>8? pump[8].c_str(): "");
		ui.Pump_label_10->setText(pump.size()>9? pump[9].c_str(): "");
		ui.Pump_label_11->setText(pump.size()>10? pump[10].c_str(): "");
		ui.Pump_label_12->setText(pump.size()>11? pump[11].c_str(): "");
	}
	catch (Tango::DevFailed &e) {
		// cout << "condServer->command_inout(GetVacuumGaugesNames);" << endl;
		Tango::Except::print_exception(e);
	}
	if (pump.size()>0) {
		ui.LowThresPressure_1->show();
		ui.HighThresPressure_1->show();
	}
	else {
		ui.LowThresPressure_1->hide();
		ui.HighThresPressure_1->hide();
	}
	if (pump.size()>1) {
		ui.LowThresPressure_2->show();
		ui.HighThresPressure_2->show();
	}
	else {
		ui.LowThresPressure_2->hide();
		ui.HighThresPressure_2->hide();
	}
	if (pump.size()>2) {
		ui.LowThresPressure_3->show();
		ui.HighThresPressure_3->show();
	}
	else {
		ui.LowThresPressure_3->hide();
		ui.HighThresPressure_3->hide();
	}
	if (pump.size()>3) {
		ui.LowThresPressure_4->show();
		ui.HighThresPressure_4->show();
	}
	else {
		ui.LowThresPressure_4->hide();
		ui.HighThresPressure_4->hide();
	}
	if (pump.size()>4) {
		ui.LowThresPressure_5->show();
		ui.HighThresPressure_5->show();
	}
	else {
		ui.LowThresPressure_5->hide();
		ui.HighThresPressure_5->hide();
	}
	if (pump.size()>5) {
		ui.LowThresPressure_6->show();
		ui.HighThresPressure_6->show();
	}
	else {
		ui.LowThresPressure_6->hide();
		ui.HighThresPressure_6->hide();
	}
	if (pump.size()>6) {
		ui.LowThresPressure_7->show();
		ui.HighThresPressure_7->show();
	}
	else {
		ui.LowThresPressure_7->hide();
		ui.HighThresPressure_7->hide();
	}
	if (pump.size()>7) {
		ui.LowThresPressure_8->show();
		ui.HighThresPressure_8->show();
	}
	else {
		ui.LowThresPressure_8->hide();
		ui.HighThresPressure_8->hide();
	}
	if (pump.size()>8) {
		ui.LowThresPressure_9->show();
		ui.HighThresPressure_9->show();
	}
	else {
		ui.LowThresPressure_9->hide();
		ui.HighThresPressure_9->hide();
	}
	if (pump.size()>9) {
		ui.LowThresPressure_10->show();
		ui.HighThresPressure_10->show();
	}
	else {
		ui.LowThresPressure_10->hide();
		ui.HighThresPressure_10->hide();
	}
	if (pump.size()>10) {
		ui.LowThresPressure_11->show();
		ui.HighThresPressure_11->show();
	}
	else {
		ui.LowThresPressure_11->hide();
		ui.HighThresPressure_11->hide();
	}
	if (pump.size()>11) {
		ui.LowThresPressure_12->show();
		ui.HighThresPressure_12->show();
	}
	else {
		ui.LowThresPressure_12->hide();
		ui.HighThresPressure_12->hide();
	}
	ui.Pressure->setNumRows(pump.size());
	ui.Pressure->resize(90, 30*pump.size());
	ui.HighLowRecoverTime->setNumRows(pump.size());
	ui.HighLowRecoverTime->resize(80, 30*pump.size());
	ui.HighThresCounter->setNumRows(pump.size());
	ui.HighThresCounter->resize(31, 30*pump.size());
	connect(ui.k00, SIGNAL(clicked()), this, SLOT(setK0()));
	if (!modNum.compare("sp") || !modNum.compare("s"))  mySetColor(ui.k00, "#ff7744", "#555588");
	ui.modStat_00->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_00->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_00->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_00->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_00->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));
	/*
	*/
	connect(ui.k01, SIGNAL(clicked()), this, SLOT(setK1()));
	if (modNum.toUInt()==1) mySetColor(ui.k01, "#ff7744", "#555588");
	ui.modStat_01->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_01->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_01->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_01->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_01->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k02, SIGNAL(clicked()), this, SLOT(setK2()));
	if (modNum.toUInt()==2) mySetColor(ui.k02, "#ff7744", "#555588");
	ui.modStat_02->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_02->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_02->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_02->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_02->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k03, SIGNAL(clicked()), this, SLOT(setK3()));
	if (modNum.toUInt()==3) mySetColor(ui.k03, "#ff7744", "#555588");
	ui.modStat_03->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_03->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_03->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_03->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_03->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k04, SIGNAL(clicked()), this, SLOT(setK4()));
	if (modNum.toUInt()==4) mySetColor(ui.k04, "#ff7744", "#555588");
	ui.modStat_04->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_04->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_04->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_04->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_04->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k05, SIGNAL(clicked()), this, SLOT(setK5()));
	if (modNum.toUInt()==5) mySetColor(ui.k05, "#ff7744", "#555588");
	ui.modStat_05->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_05->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_05->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_05->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_05->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k06, SIGNAL(clicked()), this, SLOT(setK6()));
	if (modNum.toUInt()==6) mySetColor(ui.k06, "#ff7744", "#555588");
	ui.modStat_06->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_06->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_06->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_06->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_06->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k07, SIGNAL(clicked()), this, SLOT(setK7()));
	if (modNum.toUInt()==7) mySetColor(ui.k07, "#ff7744", "#555588");
	ui.modStat_07->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_07->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_07->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_07->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_07->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k08, SIGNAL(clicked()), this, SLOT(setK8()));
	if (modNum.toUInt()==8) mySetColor(ui.k08, "#ff7744", "#555588");
	ui.modStat_08->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_08->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_08->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_08->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_08->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k09, SIGNAL(clicked()), this, SLOT(setK9()));
	if (modNum.toUInt()==9) mySetColor(ui.k09, "#ff7744", "#555588");
	ui.modStat_09->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_09->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_09->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_09->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_09->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k10, SIGNAL(clicked()), this, SLOT(setK10()));
	if (modNum.toUInt()==10) mySetColor(ui.k10, "#ff7744", "#555588");
	ui.modStat_10->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_10->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_10->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_10->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_10->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k11, SIGNAL(clicked()), this, SLOT(setK11()));
	if (modNum.toUInt()==11) mySetColor(ui.k11, "#ff7744", "#555588");
	ui.modStat_11->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_11->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_11->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_11->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_11->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k12, SIGNAL(clicked()), this, SLOT(setK12()));
	if (modNum.toUInt()==12) mySetColor(ui.k12, "#ff7744", "#555588");
	ui.modStat_12->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_12->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_12->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_12->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_12->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k13, SIGNAL(clicked()), this, SLOT(setK13()));
	if (modNum.toUInt()==13) mySetColor(ui.k13, "#ff7744", "#555588");
	ui.modStat_13->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_13->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_13->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_13->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_13->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k14, SIGNAL(clicked()), this, SLOT(setK14()));
	if (modNum.toUInt()==14) mySetColor(ui.k14, "#ff7744", "#555588");
	ui.modStat_14->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_14->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_14->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_14->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_14->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k15, SIGNAL(clicked()), this, SLOT(setK15()));
	if (modNum.toUInt()==15) mySetColor(ui.k15, "#ff7744", "#555588");
	ui.modStat_15->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_15->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_15->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_15->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_15->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	connect(ui.k16, SIGNAL(clicked()), this, SLOT(setK16()));
	if (modNum.toUInt()==16) mySetColor(ui.k16, "#ff7744", "#555588");
	ui.modStat_16->setEnumDisplay(0, 0, QString(" "), QColor(255, 255, 255));
	ui.modStat_16->setEnumDisplay(0, 1, QString(" "), QColor(100, 150, 220));
	ui.modStat_16->setEnumDisplay(0, 2, QString(" "), QColor(0, 200, 0));
	ui.modStat_16->setEnumDisplay(0, 3, QString(" "), QColor(220, 0, 0));
	ui.modStat_16->setEnumDisplay(0, 4, QString(" "), QColor(240, 200, 0));

	ui.MDK_Label->setText(modNum);
	ui.RecoveryMode->setEnumDisplay(0, "RF", Qt::white);
	ui.RecoveryMode->setEnumDisplay(1, "V", Qt::white);

	if (!modNum.compare("XS")) {
		ui.Status->setSource("kxs/mod/modcond_kxs.01/Status");
		ui.RecoveryMode->setSource("kxs/mod/modcond_kxs.01/RecoveryMode");
	}
	else {
		ui.Status->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Status");
		ui.RecoveryMode->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryMode");
	}
	ui.ConfigParam->setEnumDisplay(0, 0, QString("Slow"), QColor(255, 255, 255));
	ui.ConfigParam->setEnumDisplay(0, 1, QString("Normal"), QColor(255, 255, 255));
	ui.ConfigParam->setEnumDisplay(0, 2, QString("Fast"), QColor(255, 255, 255));
	ui.ConfigParam->setEnumDisplay(0, 3, QString("N.A."), QColor(255, 255, 255));

	connect(ui.update_PushButton, SIGNAL(clicked()), this, SLOT(updateLogs()));
	connect(ui.update_PushButton_2, SIGNAL(clicked()), this, SLOT(updateLogs()));
	ui.label_4->setPixmap(QPixmap(imgpath+"logo_fermi.png"));
	if (!modNum.compare("sp") || !modNum.compare("s")) {
		ui.On_PushButton->setTargets("ks/mod/modcond_ks.01->On;f/modulators/modulators/kg00enable(0)");
		ui.MDK_Label->setText("S");
	}
	else if (!modNum.compare("XS")) {
		ui.On_PushButton->setTargets("kxs/mod/modcond_kxs.01->On");
	}
	ui.ConfigParamSet->setCurrentIndex(3);
	if (!modNum.compare("05")) {
		configK5();
	}
	else if (!modNum.compare("ST", Qt::CaseInsensitive)) {
		configKST();
	}
	else if (!modNum.compare("XS", Qt::CaseInsensitive)) {
		configKXS();
	}
	else if (!modNum.compare("16")) {
		configK16();
	}
	//------------------------------------------------------------------------
	// RESIZE SUPPORT
	//------------------------------------------------------------------------
	// init resize data structure
	w_buffer tmp_buffer;
	// init global container
	tmp_buffer.w = NULL;
	tmp_buffer.x = 0.0;
	tmp_buffer.y = 0.0;
	tmp_buffer.width = (double) this->width();
	tmp_buffer.height = (double) this->height();
	ws_buffer.push_back(tmp_buffer);
	// init all widgets
	foreach(QWidget *w, this->findChildren<QWidget *>()) {
		if (w->objectName().length()<1) continue;
		// cout << w->objectName().toStdString().c_str() << endl;
		tmp_buffer.w = w;
		tmp_buffer.font = w->font();
		tmp_buffer.x = (double) w->x();
		tmp_buffer.y = (double) w->y();
		tmp_buffer.width = (double) w->width();
		tmp_buffer.height = (double) w->height();
		ws_buffer.push_back(tmp_buffer);
	}
	resizeFactorX = resizeFactorY = resizeFactorM = 1;
	QDesktopWidget mydesk;
	if ((mydesk.height()>100) && (this->height() > mydesk.height()-20)) {
		this->resize(this->width()*(mydesk.height()-20)/this->height(), mydesk.height()-20);
	}

}

modcond::~modcond()
{
	disconnectDevice();
	exit(0);
}
void modcond::resizeEvent( QResizeEvent * event ) {
	foreach(w_buffer tmp_buffer, ws_buffer) {
		// if (tmp_buffer.w!=NULL) { cout << tmp_buffer.w->objectName().toStdString().c_str() << endl;}
		if (tmp_buffer.w==NULL) {
			resizeFactorX = this->width()/tmp_buffer.width;
			resizeFactorY = this->height()/tmp_buffer.height;
			resizeFactorM = resizeFactorX<resizeFactorY? resizeFactorX: resizeFactorY;
		}
		else {
			tmp_buffer.w->resize(tmp_buffer.width*resizeFactorX,tmp_buffer.height*resizeFactorY);
			tmp_buffer.w->move(tmp_buffer.x*resizeFactorX,tmp_buffer.y*resizeFactorY);
			// fonts require special treatment
			QFont font = tmp_buffer.font;
			font.setPointSizeF(font.pointSizeF()*resizeFactorM);
			tmp_buffer.w->setFont(font);
		}
	}
}

int modcond::mySetColor(QPushButton *widget, const char *textColor, const char *backgroundColor)
{
	QPalette button;
	button.setColor(QPalette::WindowText, QColor(textColor));
	button.setColor(QPalette::ButtonText, QColor(textColor));
	widget->setPalette(button);
	button.setColor(QPalette::Button, QColor(backgroundColor));
	widget->setPalette(button);
	return 0;
}

void modcond::setIndexMode(bool ok)
{
/*
	printf("\e[1;32msetIndexMode()\e[0m\n");
	if(ok)
	{
		TComboBox *combo = dynamic_cast<TComboBox *>(ui.RecoveryMode->writer());
		if(combo) {
			combo->setIndexMode(true);
			//printf("\e[1;33minserisco mia lista\e[0m\n");
			//combo->insertItems(0, QStringList()<<"RF"<<"V");
		}
	}
*/
}

void modcond::updateLogs()
{
	Tango::DeviceData devData;
	vector<string> data;
	Tango::DeviceData num;
	Tango::DevLong n;
	n = 10;
	num << n;
	try {
		devData = condServer->command_inout("GetIlockEventDate", num);
		devData >> data;
		ui.Ilock->clear();
		for (unsigned i=0; i<data.size(); i++) {
			ui.Ilock->append(data[i].c_str());
		}
	}
	catch (Tango::DevFailed &e) {
		cout << "condServer->command_inout(GetIlockEventDate);" << endl;
		Tango::Except::print_exception(e);
	}
	try {
		devData = condServer->command_inout("GetHighThresEventDate", num);
		devData >> data;
		ui.HighThres->clear();
		for (unsigned i=0; i<data.size(); i++) {
			ui.HighThres->append(data[i].c_str());
		}
	}
	catch (Tango::DevFailed &e) {
		cout << "condServer->command_inout(GetHighThresEventDate);" << endl;
		Tango::Except::print_exception(e);
	}
}

void modcond::disconnectDevice()
{
	if (condServer) {
		delete condServer;
		condServer  = 0;
	}
}

int modcond::connectDevice(QString dev, Tango::DeviceProxy **srv)
{
	// cout << "connectDevice()" << endl;
	try {
		*srv = new Tango::DeviceProxy(dev.toStdString().c_str());
	}
	catch (...) {
		*srv = 0;
		sleep(1);
		try {
			*srv = new Tango::DeviceProxy(dev.toStdString().c_str());
		}
		catch (...) {
			*srv = 0;
			QMessageBox::information(0, "cannot connect "+dev+" server",
			QString("Cannot connect "+dev+" server for modulator "));
			cout << "connectDevice(), return -1" << endl;
			return -1;
		}
	}
	// cout << "connectDevice(), return 0" << endl;
	return 0;
}

int modcond::configWidgets()
{
	// ui.mod_title->setText("K"+modNum);
	Tango::DeviceProxy *oldCondServer;
	oldCondServer = condServer;
	condServer = 0;
	// cout << "configWidgets()" << endl;

	if (!modNum.compare("05")) {
		if (connectDevice("k"+modNum+"/mod/modcond_k"+modNum+".01", &condServer) < 0) {
			cout << "connectDevice()" << endl;
			condServer = oldCondServer;
			return -1;
		}
	}
	else if (!modNum.compare("ST")) {
		if (connectDevice("kst/mod/modcond_kst.01", &condServer) < 0) {
			cout << "connectDevice()" << endl;
			condServer = oldCondServer;
			return -1;
		}
	}
	else if (!modNum.compare("XS")) {
		if (connectDevice("kxs/mod/modcond_kxs.01", &condServer) < 0) {
			cout << "connectDevice()" << endl;
			condServer = oldCondServer;
			return -1;
		}
	}
	else if (!modNum.compare("16")) {
		if (connectDevice("k16/mod/modcond_k16.01", &condServer) < 0) {
			cout << "connectDevice()" << endl;
			condServer = oldCondServer;
			return -1;
		}
	}
	else if (connectDevice("k"+modNum+"/mod/modcond_k"+modNum+".01", &condServer) < 0) {
		cout << "connectDevice()" << endl;
		condServer = oldCondServer;
		return -1;
	}
	delete oldCondServer;
	cout << "connectDevice() OK" << endl;
	// patch for bug on set_source()
	foreach(QWidget *w, findChildren<QWidget *>())
	{
		QTangoComProxyReader *r = dynamic_cast<QTangoComProxyReader *>(w);
		if (w->objectName().startsWith("modStat_")) continue;
		if(r)
			r->unsetSource();
		else
		{
			QTangoComProxyWriter *wr = dynamic_cast<QTangoComProxyWriter *> (w);
			if(wr)
				wr->clearTargets();
		}
	}
	cout << "cleared OK" << endl;

	sleep(0.2);
	
	// Main tab
	Tango::DeviceData pumpData;
	vector<string> pump;
	try {
		pumpData = condServer->command_inout("GetVacuumGaugesNames");
		pumpData >> pump;
		ui.Pump_label_1->setText(pump.size()>0? pump[0].c_str(): "");
		ui.Pump_label_2->setText(pump.size()>1? pump[1].c_str(): "");
		ui.Pump_label_3->setText(pump.size()>2? pump[2].c_str(): "");
		ui.Pump_label_4->setText(pump.size()>3? pump[3].c_str(): "");
		ui.Pump_label_5->setText(pump.size()>4? pump[4].c_str(): "");
		ui.Pump_label_6->setText(pump.size()>5? pump[5].c_str(): "");
		ui.Pump_label_7->setText(pump.size()>6? pump[6].c_str(): "");
		ui.Pump_label_8->setText(pump.size()>7? pump[7].c_str(): "");
		ui.Pump_label_9->setText(pump.size()>8? pump[8].c_str(): "");
		ui.Pump_label_10->setText(pump.size()>9? pump[9].c_str(): "");
		ui.Pump_label_11->setText(pump.size()>10? pump[10].c_str(): "");
		ui.Pump_label_12->setText(pump.size()>11? pump[11].c_str(): "");
	}
	catch (Tango::DevFailed &e) {
		cout << "condServer->command_inout(GetVacuumGaugesNames);" << endl;
		Tango::Except::print_exception(e);
	}
	catch (...) {
		cout << "... condServer->command_inout(GetVacuumGaugesNames);" << endl;
	}
	if (pump.size()>0) {
		ui.LowThresPressure_1->show();
		ui.HighThresPressure_1->show();
	}
	else {
		ui.LowThresPressure_1->hide();
		ui.HighThresPressure_1->hide();
	}
	if (pump.size()>1) {
		ui.LowThresPressure_2->show();
		ui.HighThresPressure_2->show();
	}
	else {
		ui.LowThresPressure_2->hide();
		ui.HighThresPressure_2->hide();
	}
	if (pump.size()>2) {
		ui.LowThresPressure_3->show();
		ui.HighThresPressure_3->show();
	}
	else {
		ui.LowThresPressure_3->hide();
		ui.HighThresPressure_3->hide();
	}
	if (pump.size()>3) {
		ui.LowThresPressure_4->show();
		ui.HighThresPressure_4->show();
	}
	else {
		ui.LowThresPressure_4->hide();
		ui.HighThresPressure_4->hide();
	}
	if (pump.size()>4) {
		ui.LowThresPressure_5->show();
		ui.HighThresPressure_5->show();
	}
	else {
		ui.LowThresPressure_5->hide();
		ui.HighThresPressure_5->hide();
	}
	if (pump.size()>5) {
		ui.LowThresPressure_6->show();
		ui.HighThresPressure_6->show();
	}
	else {
		ui.LowThresPressure_6->hide();
		ui.HighThresPressure_6->hide();
	}
	if (pump.size()>6) {
		ui.LowThresPressure_7->show();
		ui.HighThresPressure_7->show();
	}
	else {
		ui.LowThresPressure_7->hide();
		ui.HighThresPressure_7->hide();
	}
	if (pump.size()>7) {
		ui.LowThresPressure_8->show();
		ui.HighThresPressure_8->show();
	}
	else {
		ui.LowThresPressure_8->hide();
		ui.HighThresPressure_8->hide();
	}
	if (pump.size()>8) {
		ui.LowThresPressure_9->show();
		ui.HighThresPressure_9->show();
	}
	else {
		ui.LowThresPressure_9->hide();
		ui.HighThresPressure_9->hide();
	}
	if (pump.size()>9) {
		ui.LowThresPressure_10->show();
		ui.HighThresPressure_10->show();
	}
	else {
		ui.LowThresPressure_10->hide();
		ui.HighThresPressure_10->hide();
	}
	if (pump.size()>10) {
		ui.LowThresPressure_11->show();
		ui.HighThresPressure_11->show();
	}
	else {
		ui.LowThresPressure_11->hide();
		ui.HighThresPressure_11->hide();
	}
	if (pump.size()>11) {
		ui.LowThresPressure_12->show();
		ui.HighThresPressure_12->show();
	}
	else {
		ui.LowThresPressure_12->hide();
		ui.HighThresPressure_12->hide();
	}
	ui.Pressure->setNumRows(pump.size());
	ui.Pressure->resize(90*resizeFactorX, 30*pump.size()*resizeFactorY);
	ui.HighLowRecoverTime->setNumRows(pump.size());
	ui.HighLowRecoverTime->resize(80*resizeFactorX, 30*pump.size()*resizeFactorY);
	ui.HighThresCounter->setNumRows(pump.size());
	ui.HighThresCounter->resize(31*resizeFactorX, 30*pump.size()*resizeFactorY);

	if (!modNum.compare("05")) {
		configK5();
		return 0;
	}

	if (!modNum.compare("16")) {
		configK16();
		return 0;
	}
	if (!modNum.compare("ST"), Qt::CaseInsensitive) {
		configKST();
		return 0;
	}
	if (!modNum.compare("XS"), Qt::CaseInsensitive) {
		configKXS();
		return 0;
	}
	cout << "config all" << endl; // return 0;

	if (!modNum.compare("sp") || !modNum.compare("s")) {
		ui.On_PushButton->setTargets("ks/mod/modcond_ks.01->On;f/modulators/modulators/kg00enable(0)");
	}
	else {
		ui.On_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->On;f/modulators/modulators/kg"+modNum+"enable(0)");
	}
	ui.Off_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->Off");
	ui.Standby_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->Standby");
	ui.Status->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Status");
	ui.State->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/State");

	ui.HighThresPressure_1->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure1");
	ui.HighThresPressure_2->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure2");
	ui.HighThresPressure_3->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure3");
	ui.HighThresPressure_4->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure4");
	ui.HighThresPressure_5->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure5");
	ui.HighThresPressure_6->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure6");
	ui.HighThresPressure_7->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure7");
	ui.HighThresPressure_8->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure8");
	ui.HighThresPressure_9->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure9");
	ui.HighThresPressure_10->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure10");
	ui.HighThresPressure_11->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure11");
	ui.HighThresPressure_12->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure12");

	ui.LowThresPressure_1->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure1");
	ui.LowThresPressure_2->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure2");
	ui.LowThresPressure_3->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure3");
	ui.LowThresPressure_4->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure4");
	ui.LowThresPressure_5->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure5");
	ui.LowThresPressure_6->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure6");
	ui.LowThresPressure_7->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure7");
	ui.LowThresPressure_8->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure8");
	ui.LowThresPressure_9->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure9");
	ui.LowThresPressure_10->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure10");
	ui.LowThresPressure_11->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure11");
	ui.LowThresPressure_12->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure12");

	ui.HighThresCounter->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresCounter");
	ui.Pressure->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Pressure");
	ui.HighLowRecoverTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighLowRecoverTime");
	
	// HV tab
	ui.Voltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Voltage");
	ui.VoltageChangeRatioPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageChangeRatioPerc");
	ui.MaxRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MaxRampVoltage");
	ui.MinRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MinRampVoltage");
	ui.IlockRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IlockRampVoltage");
	ui.VoltageSlowRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageSlowRampRatio");
	ui.VoltageFastRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageFastRampRatio");
	ui.VoltageDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageDecPerc");
	ui.StartRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/StartRampVoltage");
	ui.ChangeRatioVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ChangeRatioVoltage");

	// RF tab
	ui.RFLen->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFLen");
	ui.RFChangeRatioPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFChangeRatioPerc");
	ui.MaxRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MaxRampRF");
	ui.MinRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MinRampRF");
	ui.IlockRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IlockRampRF");
	ui.RFSlowRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFSlowRampRatio");
	ui.RFFastRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFFastRampRatio");
	ui.RFDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFDecPerc");
	ui.StartRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/StartRampRF");
	ui.ChangeRatioRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ChangeRatioRF");

	// Refl tab
	ui.ReflChangeRatioPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflChangeRatioPerc");
	ui.ReflDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflDecPerc");
	ui.ReflSlowRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflSlowRampRatio");
	ui.ReflFastRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflFastRampRatio");
	ui.ReflRecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflRecoveryTime");
	ui.VacuumRecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VacuumRecoveryTime");

	// Mode tab
	ui.RecoveryMode->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryMode");
	ui.RecoveryModeApply->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryMode(&RecoveryModeSet)");
	ui.RecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryTime");
	ui.WaitRampTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/WaitRampTime");
	ui.MaxAutomaticRestart->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MaxAutomaticRestart");
	ui.CurAutoRestartCounter->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/CurAutoRestartCounter");
	ui.AutoRestart->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/AutoRestart");
	ui.AutoRestart->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01/AutoRestart");
	ui.Reset_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->Reset");
	ui.ConfigParam->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Config");
	ui.ConfigParamSet->setCurrentIndex(3);
	ui.LoadConfig->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->LoadConfig(&ConfigParamSet)");
	ui.SaveConfig->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->SaveConfig(&ConfigParamSet)");
	ui.ForceSlowRatio->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->ForceSlowRatio");
	ui.ForceFastRatio->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->ForceFastRatio");
	ui.DiodeMode->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DiodeMode");
	ui.DiodeMode->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01/DiodeMode");
	ui.IpkRecoveryTimeBarrier->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IpkRecoveryTimeBarrier");
	ui.IpkRecoveryDiscount->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IpkRecoveryDiscount");
	// Cond tab
	ui.Cond1RFDec->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond1RFDec");
	ui.Cond1TimeLimit->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond1TimeLimit");
	ui.Cond2VoltageDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond2VoltageDecPerc");
	ui.Cond2RFInc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond2RFInc");
	ui.DeltaTempNormal->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DeltaTempNormal");
	ui.DeltaTempRecov->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DeltaTempRecov");
	ui.DeltaTempTimeRecov->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DeltaTempTimeRecov");

	// Log Thres tab
	ui.HighThres->clear();
	
	// Log Fault tab
	ui.Ilock->clear();
	return 0;
}

void modcond::configK5()
{
	ui.On_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->On;f/modulators/modulators/kg"+modNum+"enable(0)");
	// ui.On_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->On");
	ui.Off_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->Off");
	ui.Standby_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->Standby");
	ui.Status->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Status");
	ui.State->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/State");

	ui.HighThresPressure_1->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure1");
	ui.HighThresPressure_2->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure2");
	ui.HighThresPressure_3->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure3");
	ui.HighThresPressure_4->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure4");
	ui.HighThresPressure_5->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure5");
	ui.HighThresPressure_6->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure6");
	ui.HighThresPressure_7->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure7");
	ui.HighThresPressure_8->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure8");
	ui.HighThresPressure_9->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure9");
	ui.HighThresPressure_10->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure10");
	ui.HighThresPressure_11->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure11");
	ui.HighThresPressure_12->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresPressure12");

	ui.LowThresPressure_1->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure1");
	ui.LowThresPressure_2->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure2");
	ui.LowThresPressure_3->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure3");
	ui.LowThresPressure_4->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure4");
	ui.LowThresPressure_5->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure5");
	ui.LowThresPressure_6->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure6");
	ui.LowThresPressure_7->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure7");
	ui.LowThresPressure_8->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure8");
	ui.LowThresPressure_9->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure9");
	ui.LowThresPressure_10->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure10");
	ui.LowThresPressure_11->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure11");
	ui.LowThresPressure_12->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/LowThresPressure12");

	ui.HighThresCounter->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighThresCounter");
	ui.Pressure->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Pressure");
	ui.HighLowRecoverTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/HighLowRecoverTime");

	// HV tab
	ui.Voltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Voltage");
	ui.VoltageChangeRatioPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageChangeRatioPerc");
	ui.MaxRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MaxRampVoltage");
	ui.MinRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MinRampVoltage");
	ui.IlockRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IlockRampVoltage");
	ui.VoltageSlowRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageSlowRampRatio");
	ui.VoltageFastRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageFastRampRatio");
	ui.VoltageDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VoltageDecPerc");
	ui.StartRampVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/StartRampVoltage");
	ui.ChangeRatioVoltage->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ChangeRatioVoltage");

	// RF tab
	ui.RFLen->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFLen");
	ui.RFChangeRatioPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFChangeRatioPerc");
	ui.MaxRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MaxRampRF");
	ui.MinRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MinRampRF");
	ui.IlockRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IlockRampRF");
	ui.RFSlowRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFSlowRampRatio");
	ui.RFFastRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFFastRampRatio");
	ui.RFDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RFDecPerc");
	ui.StartRampRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/StartRampRF");
	ui.ChangeRatioRF->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ChangeRatioRF");

	ui.ReflChangeRatioPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflChangeRatioPerc");
	ui.ReflDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflDecPerc");
	ui.ReflSlowRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflSlowRampRatio");
	ui.ReflFastRampRatio->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflFastRampRatio");
	ui.ReflRecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/ReflRecoveryTime");
	ui.VacuumRecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/VacuumRecoveryTime");

	// Mode tab
	ui.RecoveryMode->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryMode");
	ui.RecoveryModeApply->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryMode(&RecoveryModeSet)");
	ui.RecoveryTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/RecoveryTime");
	ui.WaitRampTime->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/WaitRampTime");
	ui.MaxAutomaticRestart->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/MaxAutomaticRestart");
	ui.CurAutoRestartCounter->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/CurAutoRestartCounter");
	ui.AutoRestart->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/AutoRestart");
	ui.AutoRestart->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01/AutoRestart");
	ui.Reset_PushButton->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->Reset");
	ui.ConfigParam->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Config");
	ui.ConfigParamSet->setCurrentIndex(3);
	ui.LoadConfig->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->LoadConfig(&ConfigParamSet)");
	ui.SaveConfig->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->SaveConfig(&ConfigParamSet)");
	ui.ForceSlowRatio->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->ForceSlowRatio");
	ui.ForceFastRatio->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01->ForceFastRatio");
	ui.DiodeMode->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DiodeMode");
	ui.DiodeMode->setTargets("k"+modNum+"/mod/modcond_k"+modNum+".01/DiodeMode");
	ui.IpkRecoveryTimeBarrier->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IpkRecoveryTimeBarrier");
	ui.IpkRecoveryDiscount->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/IpkRecoveryDiscount");

	// Cond tab
	ui.Cond1RFDec->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond1RFDec");
	ui.Cond1TimeLimit->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond1TimeLimit");
	ui.Cond2VoltageDecPerc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond2VoltageDecPerc");
	ui.Cond2RFInc->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/Cond2RFInc");
	ui.DeltaTempNormal->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DeltaTempNormal");
	ui.DeltaTempRecov->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DeltaTempRecov");
	ui.DeltaTempTimeRecov->setSource("k"+modNum+"/mod/modcond_k"+modNum+".01/DeltaTempTimeRecov");

	// Log Thres tab
	ui.HighThres->clear();

	// Log Fault tab
	ui.Ilock->clear();

}

void modcond::configK16()
{
	ui.On_PushButton->setTargets("k16/mod/modcond_k16.01->On;f/modulators/modulators/kg"+modNum+"enable(0)");
	// ui.On_PushButton->setTargets("k16/mod/modcond_k16.01->On");
	ui.Off_PushButton->setTargets("k16/mod/modcond_k16.01->Off");
	ui.Standby_PushButton->setTargets("k16/mod/modcond_k16.01->Standby");
	ui.Status->setSource("k16/mod/modcond_k16.01/Status");
	ui.State->setSource("k16/mod/modcond_k16.01/State");

	ui.HighThresPressure_1->setSource("k16/mod/modcond_k16.01/HighThresPressure1");
	ui.HighThresPressure_2->setSource("k16/mod/modcond_k16.01/HighThresPressure2");
	ui.HighThresPressure_3->setSource("k16/mod/modcond_k16.01/HighThresPressure3");
	ui.HighThresPressure_4->setSource("k16/mod/modcond_k16.01/HighThresPressure4");
	ui.HighThresPressure_5->setSource("k16/mod/modcond_k16.01/HighThresPressure5");
	ui.HighThresPressure_6->setSource("k16/mod/modcond_k16.01/HighThresPressure6");
	ui.HighThresPressure_7->setSource("k16/mod/modcond_k16.01/HighThresPressure7");
	ui.HighThresPressure_8->setSource("k16/mod/modcond_k16.01/HighThresPressure8");
	ui.HighThresPressure_9->setSource("k16/mod/modcond_k16.01/HighThresPressure9");
	ui.HighThresPressure_10->setSource("k16/mod/modcond_k16.01/HighThresPressure10");
	ui.HighThresPressure_11->setSource("k16/mod/modcond_k16.01/HighThresPressure11");
	ui.HighThresPressure_12->setSource("k16/mod/modcond_k16.01/HighThresPressure12");

	ui.LowThresPressure_1->setSource("k16/mod/modcond_k16.01/LowThresPressure1");
	ui.LowThresPressure_2->setSource("k16/mod/modcond_k16.01/LowThresPressure2");
	ui.LowThresPressure_3->setSource("k16/mod/modcond_k16.01/LowThresPressure3");
	ui.LowThresPressure_4->setSource("k16/mod/modcond_k16.01/LowThresPressure4");
	ui.LowThresPressure_5->setSource("k16/mod/modcond_k16.01/LowThresPressure5");
	ui.LowThresPressure_6->setSource("k16/mod/modcond_k16.01/LowThresPressure6");
	ui.LowThresPressure_7->setSource("k16/mod/modcond_k16.01/LowThresPressure7");
	ui.LowThresPressure_8->setSource("k16/mod/modcond_k16.01/LowThresPressure8");
	ui.LowThresPressure_9->setSource("k16/mod/modcond_k16.01/LowThresPressure9");
	ui.LowThresPressure_10->setSource("k16/mod/modcond_k16.01/LowThresPressure10");
	ui.LowThresPressure_11->setSource("k16/mod/modcond_k16.01/LowThresPressure11");
	ui.LowThresPressure_12->setSource("k16/mod/modcond_k16.01/LowThresPressure12");

	ui.HighThresCounter->setSource("k16/mod/modcond_k16.01/HighThresCounter");
	ui.Pressure->setSource("k16/mod/modcond_k16.01/Pressure");
	ui.HighLowRecoverTime->setSource("k16/mod/modcond_k16.01/HighLowRecoverTime");

	// HV tab
	ui.Voltage->setSource("k16/mod/modcond_k16.01/Voltage");
	ui.VoltageChangeRatioPerc->setSource("k16/mod/modcond_k16.01/VoltageChangeRatioPerc");
	ui.MaxRampVoltage->setSource("k16/mod/modcond_k16.01/MaxRampVoltage");
	ui.MinRampVoltage->setSource("k16/mod/modcond_k16.01/MinRampVoltage");
	ui.IlockRampVoltage->setSource("k16/mod/modcond_k16.01/IlockRampVoltage");
	ui.VoltageSlowRampRatio->setSource("k16/mod/modcond_k16.01/VoltageSlowRampRatio");
	ui.VoltageFastRampRatio->setSource("k16/mod/modcond_k16.01/VoltageFastRampRatio");
	ui.VoltageDecPerc->setSource("k16/mod/modcond_k16.01/VoltageDecPerc");
	ui.StartRampVoltage->setSource("k16/mod/modcond_k16.01/StartRampVoltage");
	ui.ChangeRatioVoltage->setSource("k16/mod/modcond_k16.01/ChangeRatioVoltage");

	// RF tab
	ui.RFLen->setSource("k16/mod/modcond_k16.01/RFLen");
	ui.RFChangeRatioPerc->setSource("k16/mod/modcond_k16.01/RFChangeRatioPerc");
	ui.MaxRampRF->setSource("k16/mod/modcond_k16.01/MaxRampRF");
	ui.MinRampRF->setSource("k16/mod/modcond_k16.01/MinRampRF");
	ui.IlockRampRF->setSource("k16/mod/modcond_k16.01/IlockRampRF");
	ui.RFSlowRampRatio->setSource("k16/mod/modcond_k16.01/RFSlowRampRatio");
	ui.RFFastRampRatio->setSource("k16/mod/modcond_k16.01/RFFastRampRatio");
	ui.RFDecPerc->setSource("k16/mod/modcond_k16.01/RFDecPerc");
	ui.StartRampRF->setSource("k16/mod/modcond_k16.01/StartRampRF");
	ui.ChangeRatioRF->setSource("k16/mod/modcond_k16.01/ChangeRatioRF");

	ui.ReflChangeRatioPerc->setSource("k16/mod/modcond_k16.01/ReflChangeRatioPerc");
	ui.ReflDecPerc->setSource("k16/mod/modcond_k16.01/ReflDecPerc");
	ui.ReflSlowRampRatio->setSource("k16/mod/modcond_k16.01/ReflSlowRampRatio");
	ui.ReflFastRampRatio->setSource("k16/mod/modcond_k16.01/ReflFastRampRatio");
	ui.ReflRecoveryTime->setSource("k16/mod/modcond_k16.01/ReflRecoveryTime");
	ui.VacuumRecoveryTime->setSource("k16/mod/modcond_k16.01/VacuumRecoveryTime");

	// Mode tab
	ui.RecoveryMode->setSource("k16/mod/modcond_k16.01/RecoveryMode");
	ui.RecoveryModeApply->setTargets("k16/mod/modcond_k16.01/RecoveryMode(&RecoveryModeSet)");
	ui.RecoveryTime->setSource("k16/mod/modcond_k16.01/RecoveryTime");
	ui.WaitRampTime->setSource("k16/mod/modcond_k16.01/WaitRampTime");
	ui.MaxAutomaticRestart->setSource("k16/mod/modcond_k16.01/MaxAutomaticRestart");
	ui.CurAutoRestartCounter->setSource("k16/mod/modcond_k16.01/CurAutoRestartCounter");
	ui.AutoRestart->setSource("k16/mod/modcond_k16.01/AutoRestart");
	ui.AutoRestart->setTargets("k16/mod/modcond_k16.01/AutoRestart");
	ui.Reset_PushButton->setTargets("k16/mod/modcond_k16.01->Reset");
	ui.ConfigParam->setSource("k16/mod/modcond_k16.01/Config");
	ui.ConfigParamSet->setCurrentIndex(3);
	ui.LoadConfig->setTargets("k16/mod/modcond_k16.01->LoadConfig(&ConfigParamSet)");
	ui.SaveConfig->setTargets("k16/mod/modcond_k16.01->SaveConfig(&ConfigParamSet)");
	ui.ForceSlowRatio->setTargets("k16/mod/modcond_k16.01->ForceSlowRatio");
	ui.ForceFastRatio->setTargets("k16/mod/modcond_k16.01->ForceFastRatio");
	ui.DiodeMode->setSource("k16/mod/modcond_k16.01/DiodeMode");
	ui.DiodeMode->setTargets("k16/mod/modcond_k16.01/DiodeMode");
	ui.IpkRecoveryTimeBarrier->setSource("k16/mod/modcond_k16.01/IpkRecoveryTimeBarrier");
	ui.IpkRecoveryDiscount->setSource("k16/mod/modcond_k16.01/IpkRecoveryDiscount");

	// Cond tab
	ui.Cond1RFDec->setSource("k16/mod/modcond_k16.01/Cond1RFDec");
	ui.Cond1TimeLimit->setSource("k16/mod/modcond_k16.01/Cond1TimeLimit");
	ui.Cond2VoltageDecPerc->setSource("k16/mod/modcond_k16.01/Cond2VoltageDecPerc");
	ui.Cond2RFInc->setSource("k16/mod/modcond_k16.01/Cond2RFInc");
	ui.DeltaTempNormal->setSource("k16/mod/modcond_k16.01/DeltaTempNormal");
	ui.DeltaTempRecov->setSource("k16/mod/modcond_k16.01/DeltaTempRecov");
	ui.DeltaTempTimeRecov->setSource("k16/mod/modcond_k16.01/DeltaTempTimeRecov");

	// Log Thres tab
	ui.HighThres->clear();

	// Log Fault tab
	ui.Ilock->clear();

}

void modcond::configKST()
{
	ui.On_PushButton->setTargets("kst/mod/modcond_kst.01->On");
	ui.Off_PushButton->setTargets("kst/mod/modcond_kst.01->Off");
	ui.Standby_PushButton->setTargets("kst/mod/modcond_kst.01->Standby");
	ui.Status->setSource("kst/mod/modcond_kst.01/Status");
	ui.State->setSource("kst/mod/modcond_kst.01/State");

	ui.HighThresPressure_1->setSource("kst/mod/modcond_kst.01/HighThresPressure1");
	ui.HighThresPressure_2->setSource("kst/mod/modcond_kst.01/HighThresPressure2");
	ui.HighThresPressure_3->setSource("kst/mod/modcond_kst.01/HighThresPressure3");
	ui.HighThresPressure_4->setSource("kst/mod/modcond_kst.01/HighThresPressure4");
	ui.HighThresPressure_5->setSource("kst/mod/modcond_kst.01/HighThresPressure5");
	ui.HighThresPressure_6->setSource("kst/mod/modcond_kst.01/HighThresPressure6");
	ui.HighThresPressure_7->setSource("kst/mod/modcond_kst.01/HighThresPressure7");
	ui.HighThresPressure_8->setSource("kst/mod/modcond_kst.01/HighThresPressure8");
	ui.HighThresPressure_9->setSource("kst/mod/modcond_kst.01/HighThresPressure9");
	ui.HighThresPressure_10->setSource("kst/mod/modcond_kst.01/HighThresPressure10");
	ui.HighThresPressure_11->setSource("kst/mod/modcond_kst.01/HighThresPressure11");
	ui.HighThresPressure_12->setSource("kst/mod/modcond_kst.01/HighThresPressure12");

	ui.LowThresPressure_1->setSource("kst/mod/modcond_kst.01/LowThresPressure1");
	ui.LowThresPressure_2->setSource("kst/mod/modcond_kst.01/LowThresPressure2");
	ui.LowThresPressure_3->setSource("kst/mod/modcond_kst.01/LowThresPressure3");
	ui.LowThresPressure_4->setSource("kst/mod/modcond_kst.01/LowThresPressure4");
	ui.LowThresPressure_5->setSource("kst/mod/modcond_kst.01/LowThresPressure5");
	ui.LowThresPressure_6->setSource("kst/mod/modcond_kst.01/LowThresPressure6");
	ui.LowThresPressure_7->setSource("kst/mod/modcond_kst.01/LowThresPressure7");
	ui.LowThresPressure_8->setSource("kst/mod/modcond_kst.01/LowThresPressure8");
	ui.LowThresPressure_9->setSource("kst/mod/modcond_kst.01/LowThresPressure9");
	ui.LowThresPressure_10->setSource("kst/mod/modcond_kst.01/LowThresPressure10");
	ui.LowThresPressure_11->setSource("kst/mod/modcond_kst.01/LowThresPressure11");
	ui.LowThresPressure_12->setSource("kst/mod/modcond_kst.01/LowThresPressure12");

	ui.HighThresCounter->setSource("kst/mod/modcond_kst.01/HighThresCounter");
	ui.Pressure->setSource("kst/mod/modcond_kst.01/Pressure");
	ui.HighLowRecoverTime->setSource("kst/mod/modcond_kst.01/HighLowRecoverTime");

	// HV tab
	ui.Voltage->setSource("kst/mod/modcond_kst.01/Voltage");
	ui.VoltageChangeRatioPerc->setSource("kst/mod/modcond_kst.01/VoltageChangeRatioPerc");
	ui.MaxRampVoltage->setSource("kst/mod/modcond_kst.01/MaxRampVoltage");
	ui.MinRampVoltage->setSource("kst/mod/modcond_kst.01/MinRampVoltage");
	ui.IlockRampVoltage->setSource("kst/mod/modcond_kst.01/IlockRampVoltage");
	ui.VoltageSlowRampRatio->setSource("kst/mod/modcond_kst.01/VoltageSlowRampRatio");
	ui.VoltageFastRampRatio->setSource("kst/mod/modcond_kst.01/VoltageFastRampRatio");
	ui.VoltageDecPerc->setSource("kst/mod/modcond_kst.01/VoltageDecPerc");
	ui.StartRampVoltage->setSource("kst/mod/modcond_kst.01/StartRampVoltage");
	ui.ChangeRatioVoltage->setSource("kst/mod/modcond_kst.01/ChangeRatioVoltage");

	// RF tab
	ui.RFLen->setSource("kst/mod/modcond_kst.01/RFLen");
	ui.RFChangeRatioPerc->setSource("kst/mod/modcond_kst.01/RFChangeRatioPerc");
	ui.MaxRampRF->setSource("kst/mod/modcond_kst.01/MaxRampRF");
	ui.MinRampRF->setSource("kst/mod/modcond_kst.01/MinRampRF");
	ui.IlockRampRF->setSource("kst/mod/modcond_kst.01/IlockRampRF");
	ui.RFSlowRampRatio->setSource("kst/mod/modcond_kst.01/RFSlowRampRatio");
	ui.RFFastRampRatio->setSource("kst/mod/modcond_kst.01/RFFastRampRatio");
	ui.RFDecPerc->setSource("kst/mod/modcond_kst.01/RFDecPerc");
	ui.StartRampRF->setSource("kst/mod/modcond_kst.01/StartRampRF");
	ui.ChangeRatioRF->setSource("kst/mod/modcond_kst.01/ChangeRatioRF");

	ui.ReflChangeRatioPerc->setSource("kst/mod/modcond_kst.01/ReflChangeRatioPerc");
	ui.ReflDecPerc->setSource("kst/mod/modcond_kst.01/ReflDecPerc");
	ui.ReflSlowRampRatio->setSource("kst/mod/modcond_kst.01/ReflSlowRampRatio");
	ui.ReflFastRampRatio->setSource("kst/mod/modcond_kst.01/ReflFastRampRatio");
	ui.ReflRecoveryTime->setSource("kst/mod/modcond_kst.01/ReflRecoveryTime");
	ui.VacuumRecoveryTime->setSource("kst/mod/modcond_kst.01/VacuumRecoveryTime");
	/**/
	// Refl tab
	if (qApp->arguments().size()>2 && qApp->arguments()[2].contains("refl")) {
		ui.ReflChangeRatioPerc->setSource("kst/mod/modcond_kst.01/ReflChangeRatioPerc");
		ui.ReflDecPerc->setSource("kst/mod/modcond_kst.01/ReflDecPerc");
		ui.ReflSlowRampRatio->setSource("kst/mod/modcond_kst.01/ReflSlowRampRatio");
		ui.ReflFastRampRatio->setSource("kst/mod/modcond_kst.01/ReflFastRampRatio");
		ui.ReflRecoveryTime->setSource("kst/mod/modcond_kst.01/ReflRecoveryTime");
		ui.VacuumRecoveryTime->setSource("kst/mod/modcond_kst.01/VacuumRecoveryTime");
		ui.VacuumRecoveryTime->show();
		ui.VacuumRecoveryTime_label->show();
		ui.VacuumRecoveryTime_unit->show();
	}
	else {
		ui.VacuumRecoveryTime->hide();
		ui.VacuumRecoveryTime_label->hide();
		ui.VacuumRecoveryTime_unit->hide();
	}


	// Mode tab
	ui.RecoveryMode->setSource("kst/mod/modcond_kst.01/RecoveryMode");
	ui.RecoveryModeApply->setTargets("kst/mod/modcond_kst.01/RecoveryMode(&RecoveryModeSet)");
	ui.RecoveryTime->setSource("kst/mod/modcond_kst.01/RecoveryTime");
	ui.WaitRampTime->setSource("kst/mod/modcond_kst.01/WaitRampTime");
	ui.MaxAutomaticRestart->setSource("kst/mod/modcond_kst.01/MaxAutomaticRestart");
	ui.CurAutoRestartCounter->setSource("kst/mod/modcond_kst.01/CurAutoRestartCounter");
	ui.AutoRestart->setSource("kst/mod/modcond_kst.01/AutoRestart");
	ui.AutoRestart->setTargets("kst/mod/modcond_kst.01/AutoRestart");
	ui.Reset_PushButton->setTargets("kst/mod/modcond_kst.01->Reset");
	ui.ConfigParam->setSource("kst/mod/modcond_kst.01/Config");
	ui.ConfigParamSet->setCurrentIndex(3);
	ui.LoadConfig->setTargets("kst/mod/modcond_kst.01->LoadConfig(&ConfigParamSet)");
	ui.SaveConfig->setTargets("kst/mod/modcond_kst.01->SaveConfig(&ConfigParamSet)");
	ui.ForceSlowRatio->setTargets("kst/mod/modcond_kst.01->ForceSlowRatio");
	ui.ForceFastRatio->setTargets("kst/mod/modcond_kst.01->ForceFastRatio");
	ui.DiodeMode->setSource("kst/mod/modcond_kst.01/DiodeMode");
	ui.DiodeMode->setTargets("kst/mod/modcond_kst.01/DiodeMode");
	ui.IpkRecoveryTimeBarrier->setSource("kst/mod/modcond_kst.01/IpkRecoveryTimeBarrier");
	ui.IpkRecoveryDiscount->setSource("kst/mod/modcond_kst.01/IpkRecoveryDiscount");

	// Cond tab
	ui.Cond1RFDec->setSource("kst/mod/modcond_kst.01/Cond1RFDec");
	ui.Cond1TimeLimit->setSource("kst/mod/modcond_kst.01/Cond1TimeLimit");
	ui.Cond2VoltageDecPerc->setSource("kst/mod/modcond_kst.01/Cond2VoltageDecPerc");
	ui.Cond2RFInc->setSource("kst/mod/modcond_kst.01/Cond2RFInc");
	ui.DeltaTempNormal->setSource("kst/mod/modcond_kst.01/DeltaTempNormal");
	ui.DeltaTempRecov->setSource("kst/mod/modcond_kst.01/DeltaTempRecov");
	ui.DeltaTempTimeRecov->setSource("kst/mod/modcond_kst.01/DeltaTempTimeRecov");

	// Log Thres tab
	ui.HighThres->clear();

	// Log Fault tab
	ui.Ilock->clear();

}

void modcond::configKXS()
{
	ui.On_PushButton->setTargets("kxs/mod/modcond_kxs.01->On");
	ui.Off_PushButton->setTargets("kxs/mod/modcond_kxs.01->Off");
	ui.Standby_PushButton->setTargets("kxs/mod/modcond_kxs.01->Standby");
	ui.Status->setSource("kxs/mod/modcond_kxs.01/Status");
	ui.State->setSource("kxs/mod/modcond_kxs.01/State");

	ui.HighThresPressure_1->setSource("kxs/mod/modcond_kxs.01/HighThresPressure1");
	ui.HighThresPressure_2->setSource("kxs/mod/modcond_kxs.01/HighThresPressure2");
	ui.HighThresPressure_3->setSource("kxs/mod/modcond_kxs.01/HighThresPressure3");
	ui.HighThresPressure_4->setSource("kxs/mod/modcond_kxs.01/HighThresPressure4");
	ui.HighThresPressure_5->setSource("kxs/mod/modcond_kxs.01/HighThresPressure5");
	ui.HighThresPressure_6->setSource("kxs/mod/modcond_kxs.01/HighThresPressure6");
	ui.HighThresPressure_7->setSource("kxs/mod/modcond_kxs.01/HighThresPressure7");
	ui.HighThresPressure_8->setSource("kxs/mod/modcond_kxs.01/HighThresPressure8");
	ui.HighThresPressure_9->setSource("kxs/mod/modcond_kxs.01/HighThresPressure9");
	ui.HighThresPressure_10->setSource("kxs/mod/modcond_kxs.01/HighThresPressure10");
	ui.HighThresPressure_11->setSource("kxs/mod/modcond_kxs.01/HighThresPressure11");
	ui.HighThresPressure_12->setSource("kxs/mod/modcond_kxs.01/HighThresPressure12");

	ui.LowThresPressure_1->setSource("kxs/mod/modcond_kxs.01/LowThresPressure1");
	ui.LowThresPressure_2->setSource("kxs/mod/modcond_kxs.01/LowThresPressure2");
	ui.LowThresPressure_3->setSource("kxs/mod/modcond_kxs.01/LowThresPressure3");
	ui.LowThresPressure_4->setSource("kxs/mod/modcond_kxs.01/LowThresPressure4");
	ui.LowThresPressure_5->setSource("kxs/mod/modcond_kxs.01/LowThresPressure5");
	ui.LowThresPressure_6->setSource("kxs/mod/modcond_kxs.01/LowThresPressure6");
	ui.LowThresPressure_7->setSource("kxs/mod/modcond_kxs.01/LowThresPressure7");
	ui.LowThresPressure_8->setSource("kxs/mod/modcond_kxs.01/LowThresPressure8");
	ui.LowThresPressure_9->setSource("kxs/mod/modcond_kxs.01/LowThresPressure9");
	ui.LowThresPressure_10->setSource("kxs/mod/modcond_kxs.01/LowThresPressure10");
	ui.LowThresPressure_11->setSource("kxs/mod/modcond_kxs.01/LowThresPressure11");
	ui.LowThresPressure_12->setSource("kxs/mod/modcond_kxs.01/LowThresPressure12");

	ui.HighThresCounter->setSource("kxs/mod/modcond_kxs.01/HighThresCounter");
	ui.Pressure->setSource("kxs/mod/modcond_kxs.01/Pressure");
	ui.HighLowRecoverTime->setSource("kxs/mod/modcond_kxs.01/HighLowRecoverTime");

	// HV tab
	ui.Voltage->setSource("kxs/mod/modcond_kxs.01/Voltage");
	ui.VoltageChangeRatioPerc->setSource("kxs/mod/modcond_kxs.01/VoltageChangeRatioPerc");
	ui.MaxRampVoltage->setSource("kxs/mod/modcond_kxs.01/MaxRampVoltage");
	ui.MinRampVoltage->setSource("kxs/mod/modcond_kxs.01/MinRampVoltage");
	ui.IlockRampVoltage->setSource("kxs/mod/modcond_kxs.01/IlockRampVoltage");
	ui.VoltageSlowRampRatio->setSource("kxs/mod/modcond_kxs.01/VoltageSlowRampRatio");
	ui.VoltageFastRampRatio->setSource("kxs/mod/modcond_kxs.01/VoltageFastRampRatio");
	ui.VoltageDecPerc->setSource("kxs/mod/modcond_kxs.01/VoltageDecPerc");
	ui.StartRampVoltage->setSource("kxs/mod/modcond_kxs.01/StartRampVoltage");
	ui.ChangeRatioVoltage->setSource("kxs/mod/modcond_kxs.01/ChangeRatioVoltage");

	// RF tab
	ui.RFLen->setSource("kxs/mod/modcond_kxs.01/RFLen");
	ui.RFChangeRatioPerc->setSource("kxs/mod/modcond_kxs.01/RFChangeRatioPerc");
	ui.MaxRampRF->setSource("kxs/mod/modcond_kxs.01/MaxRampRF");
	ui.MinRampRF->setSource("kxs/mod/modcond_kxs.01/MinRampRF");
	ui.IlockRampRF->setSource("kxs/mod/modcond_kxs.01/IlockRampRF");
	ui.RFSlowRampRatio->setSource("kxs/mod/modcond_kxs.01/RFSlowRampRatio");
	ui.RFFastRampRatio->setSource("kxs/mod/modcond_kxs.01/RFFastRampRatio");
	ui.RFDecPerc->setSource("kxs/mod/modcond_kxs.01/RFDecPerc");
	ui.StartRampRF->setSource("kxs/mod/modcond_kxs.01/StartRampRF");
	ui.ChangeRatioRF->setSource("kxs/mod/modcond_kxs.01/ChangeRatioRF");

	ui.ReflChangeRatioPerc->setSource("kxs/mod/modcond_kxs.01/ReflChangeRatioPerc");
	ui.ReflDecPerc->setSource("kxs/mod/modcond_kxs.01/ReflDecPerc");
	ui.ReflSlowRampRatio->setSource("kxs/mod/modcond_kxs.01/ReflSlowRampRatio");
	ui.ReflFastRampRatio->setSource("kxs/mod/modcond_kxs.01/ReflFastRampRatio");
	ui.ReflRecoveryTime->setSource("kxs/mod/modcond_kxs.01/ReflRecoveryTime");
	ui.VacuumRecoveryTime->setSource("kxs/mod/modcond_kxs.01/VacuumRecoveryTime");
	/**/
	// Refl tab
	if (qApp->arguments().size()>2 && qApp->arguments()[2].contains("refl")) {
		ui.ReflChangeRatioPerc->setSource("kxs/mod/modcond_kxs.01/ReflChangeRatioPerc");
		ui.ReflDecPerc->setSource("kxs/mod/modcond_kxs.01/ReflDecPerc");
		ui.ReflSlowRampRatio->setSource("kxs/mod/modcond_kxs.01/ReflSlowRampRatio");
		ui.ReflFastRampRatio->setSource("kxs/mod/modcond_kxs.01/ReflFastRampRatio");
		ui.ReflRecoveryTime->setSource("kxs/mod/modcond_kxs.01/ReflRecoveryTime");
		ui.VacuumRecoveryTime->setSource("kxs/mod/modcond_kxs.01/VacuumRecoveryTime");
		ui.VacuumRecoveryTime->show();
		ui.VacuumRecoveryTime_label->show();
		ui.VacuumRecoveryTime_unit->show();
	}
	else {
		ui.VacuumRecoveryTime->hide();
		ui.VacuumRecoveryTime_label->hide();
		ui.VacuumRecoveryTime_unit->hide();
	}


	// Mode tab
	ui.RecoveryMode->setSource("kxs/mod/modcond_kxs.01/RecoveryMode");
	ui.RecoveryModeApply->setTargets("kxs/mod/modcond_kxs.01/RecoveryMode(&RecoveryModeSet)");
	ui.RecoveryTime->setSource("kxs/mod/modcond_kxs.01/RecoveryTime");
	ui.WaitRampTime->setSource("kxs/mod/modcond_kxs.01/WaitRampTime");
	ui.MaxAutomaticRestart->setSource("kxs/mod/modcond_kxs.01/MaxAutomaticRestart");
	ui.CurAutoRestartCounter->setSource("kxs/mod/modcond_kxs.01/CurAutoRestartCounter");
	ui.AutoRestart->setSource("kxs/mod/modcond_kxs.01/AutoRestart");
	ui.AutoRestart->setTargets("kxs/mod/modcond_kxs.01/AutoRestart");
	ui.Reset_PushButton->setTargets("kxs/mod/modcond_kxs.01->Reset");
	ui.ConfigParam->setSource("kxs/mod/modcond_kxs.01/Config");
	ui.ConfigParamSet->setCurrentIndex(3);
	ui.LoadConfig->setTargets("kxs/mod/modcond_kxs.01->LoadConfig(&ConfigParamSet)");
	ui.SaveConfig->setTargets("kxs/mod/modcond_kxs.01->SaveConfig(&ConfigParamSet)");
	ui.ForceSlowRatio->setTargets("kxs/mod/modcond_kxs.01->ForceSlowRatio");
	ui.ForceFastRatio->setTargets("kxs/mod/modcond_kxs.01->ForceFastRatio");
	ui.DiodeMode->setSource("kxs/mod/modcond_kxs.01/DiodeMode");
	ui.DiodeMode->setTargets("kxs/mod/modcond_kxs.01/DiodeMode");
	ui.IpkRecoveryTimeBarrier->setSource("kxs/mod/modcond_kxs.01/IpkRecoveryTimeBarrier");
	ui.IpkRecoveryDiscount->setSource("kxs/mod/modcond_kxs.01/IpkRecoveryDiscount");

	// Cond tab
	ui.Cond1RFDec->setSource("kxs/mod/modcond_kxs.01/Cond1RFDec");
	ui.Cond1TimeLimit->setSource("kxs/mod/modcond_kxs.01/Cond1TimeLimit");
	ui.Cond2VoltageDecPerc->setSource("kxs/mod/modcond_kxs.01/Cond2VoltageDecPerc");
	ui.Cond2RFInc->setSource("kxs/mod/modcond_kxs.01/Cond2RFInc");
	ui.DeltaTempNormal->setSource("kxs/mod/modcond_kxs.01/DeltaTempNormal");
	ui.DeltaTempRecov->setSource("kxs/mod/modcond_kxs.01/DeltaTempRecov");
	ui.DeltaTempTimeRecov->setSource("kxs/mod/modcond_kxs.01/DeltaTempTimeRecov");

	// Log Thres tab
	ui.HighThres->clear();

	// Log Fault tab
	ui.Ilock->clear();

}

void modcond::setMod(int mod)
{
	cout << "setMod(" << mod << ")" << endl; // return;
	char modStr[4];
	oldModNum = modNum;
	if (mod==0) {
		modNum = "s";
	}
	else {
		sprintf(modStr, "%02d", mod);
		modNum = modStr;
	}
	cout << " setMod(" << modStr << ")" << endl;

	mySetColor(ui.k00, !modNum.compare("sp") || !modNum.compare("s")? "#ff7744": "#000000", !modNum.compare("sp") || !modNum.compare("s")? "#555588": "#dddddd");
	mySetColor(ui.k01, mod==1? "#ff7744": "#000000", mod==1? "#555588": "#dddddd");
	mySetColor(ui.k02, mod==2? "#ff7744": "#000000", mod==2? "#555588": "#dddddd");
	mySetColor(ui.k03, mod==3? "#ff7744": "#000000", mod==3? "#555588": "#dddddd");
	mySetColor(ui.k04, mod==4? "#ff7744": "#000000", mod==4? "#555588": "#dddddd");
	mySetColor(ui.k05, mod==5? "#ff7744": "#000000", mod==5? "#555588": "#dddddd");
	mySetColor(ui.k06, mod==6? "#ff7744": "#000000", mod==6? "#555588": "#dddddd");
	mySetColor(ui.k07, mod==7? "#ff7744": "#000000", mod==7? "#555588": "#dddddd");
	mySetColor(ui.k08, mod==8? "#ff7744": "#000000", mod==8? "#555588": "#dddddd");
	mySetColor(ui.k09, mod==9? "#ff7744": "#000000", mod==9? "#555588": "#dddddd");
	mySetColor(ui.k10, mod==10? "#ff7744": "#000000", mod==10? "#555588": "#dddddd");
	mySetColor(ui.k11, mod==11? "#ff7744": "#000000", mod==11? "#555588": "#dddddd");
	mySetColor(ui.k12, mod==12? "#ff7744": "#000000", mod==12? "#555588": "#dddddd");
	mySetColor(ui.k13, mod==13? "#ff7744": "#000000", mod==13? "#555588": "#dddddd");
	mySetColor(ui.k14, mod==14? "#ff7744": "#000000", mod==14? "#555588": "#dddddd");
	mySetColor(ui.k15, mod==15? "#ff7744": "#000000", mod==15? "#555588": "#dddddd");
	mySetColor(ui.k16, mod==16? "#ff7744": "#000000", mod==16? "#555588": "#dddddd");

	if (!modNum.compare("sp") || !modNum.compare("s")) {
		ui.MDK_Label->setText("S");
	}
	else {
		ui.MDK_Label->setText(modNum);
	}
	
	if (configWidgets() < 0) {
		cout << "configWidgets() error" << endl;
		modNum = oldModNum;
		return;
	}
	updateLogs();
}

void modcond::setK0() {setMod(0);}
void modcond::setK1() {setMod(1);}
void modcond::setK2() {setMod(2);}
void modcond::setK3() {setMod(3);}
void modcond::setK4() {setMod(4);}
void modcond::setK5() {setMod(5);}
void modcond::setK6() {setMod(6);}
void modcond::setK7() {setMod(7);}
void modcond::setK8() {setMod(8);}
void modcond::setK9() {setMod(9);}
void modcond::setK10() {setMod(10);}
void modcond::setK11() {setMod(11);}
void modcond::setK12() {setMod(12);}
void modcond::setK13() {setMod(13);}
void modcond::setK14() {setMod(14);}
void modcond::setK15() {setMod(15);}
void modcond::setK16() {setMod(16);}
